///////////////////////////////////////////////////////////////////////////////
//
//  File Name
//      EXCHHOOK.CPP 
//
//      IExchExt interface methods:
//          MyExchExt::QueryInterface()
//          MyExchExt::AddRef()
//          MyExchExt::Release()
//          MyExchExt::Install()
//
//      IExchExtPropertySheets interface methods:
//          IExchExtPropertySheets::QueryInterface()
//          IExchExtPropertySheets::AddRef()
//          IExchExtPropertySheets::Release()
//
//          IExchExtPropertySheets::GetMaxPageCount()
//          IExchExtPropertySheets::GetPages()
//          IExchExtPropertySheets::FreePages()
//
//      IExchExtMessageEvents interface methods:
//          IExchExtMessageEvents::QueryInterface()
//          IExchExtMessageEvents::AddRef()
//          IExchExtMessageEvents::Release()
//
//          IExchExtMessageEvents::OnCheckNames()
//          IExchExtMessageEvents::OnCheckNamesComplete()
//          IExchExtMessageEvents::OnRead()
//          IExchExtMessageEvents::OnReadComplete()
//          IExchExtMessageEvents::OnSubmit()
//          IExchExtMessageEvents::OnSubmitComplete()
//          IExchExtMessageEvents::OnWrite()
//          IExchExtMessageEvents::OnWriteComplete()
//
//      IOutlookExtItemEvents interface methods:
//          IOutlookExtItemEvents::QueryInterface()
//          IOutlookExtItemEvents::AddRef()
//          IOutlookExtItemEvents::Release()
//
//          IOutlookExtItemEvents::OnOpen()
//          IOutlookExtItemEvents::OnOpenComplete()
//          IOutlookExtItemEvents::OnClose()
//          IOutlookExtItemEvents::OnCloseComplete()
//
#define INITGUID
#define USES_IID_IExchExt
#define USES_IID_IExchExtAdvancedCriteria
#define USES_IID_IExchExtAttachedFileEvents
#define USES_IID_IExchExtCommands
#define USES_IID_IExchExtMessageEvents
#define USES_IID_IExchExtPropertySheets
#define USES_IID_IExchExtSessionEvents
#define USES_IID_IExchExtUserEvents
#define USES_IID_IMessage
#define USES_PS_MAPI
#define USES_PS_PUBLIC_STRINGS
#define cbStrLenA(sz)   ((lstrlenA((sz)) + 1) * sizeof(CHAR))
#define cbStrLenW(sz)   ((lstrlenW((sz)) + 1) * sizeof(WCHAR))

#include "EXCHHOOK.H"
#include <MAPIGUID.H>
#include <SHELLAPI.H>

///////////////////////////////////////////////////////////////////////////////
//    global data for DLL
HINSTANCE ghInstDLL = NULL;  // instance handle of DLL

///////////////////////////////////////////////////////////////////////////////
//    global data that supports extension functionality
char szMessageBoxTitle[40] = TEXT ( "Message Handling" ) ;
char szQuote[2] = TEXT ( "\"" ) ;
char szSpace[2] = TEXT ( " " ) ;

DWORD	dwPreventOpen = 1 ;
DWORD	dwConfirmDialog = 1 ;
DWORD	dwDebug = 0 ;

LPMAPISESSION	lpSession ;
LPADRBOOK		lpAdrbook ;

char	szRootKey[80] = TEXT ( "SOFTWARE\\ExchangeHook\\Message Handlers" ) ;
char	szHandlerName[40] = TEXT ( "" ) ;
LPTSTR	lpszSenderEmail = NULL;
char	szSenderEmailAddress[SenderEmailAddressSize] = TEXT ( "emailid@domain.com" ) ;
char	szFilePath[FilePathSize] = TEXT ( "\"Specify a path and file name\"" ) ;

///////////////////////////////////////////////////////////////////////////////
//    FUNCTION: DLLMain()
//
//    Purpose
//    Do initilization processesing
//
//    Return Value
//    TRUE - DLL successfully loads and LoadLibrary will succeed.
//    FALSE - will cause an Exchange error message saying it cannot locate
//            the extension DLL.
//
BOOL WINAPI DllMain(
    HINSTANCE  hinstDLL,
    DWORD  fdwReason,   
    LPVOID  lpvReserved) 
{
 if (DLL_PROCESS_ATTACH == fdwReason)
 {
    ghInstDLL = hinstDLL;
 }

 return TRUE;

}


///////////////////////////////////////////////////////////////////////////////
//    FUNCTION: ExchEntryPoint
//
//    Parameters - none
//
//    Purpose
//    The entry point called by Exchange.
//
//    Return Value
//    Pointer to Exchange Extension (IExchExt) interface
//
//    Comments
//    Exchange Client calls this for each context entry.  
//

LPEXCHEXT CALLBACK ExchEntryPoint(void)
{
     return new MyExchExt;
}


///////////////////////////////////////////////////////////////////////////////
//    MyExchExt::MyExchExt()
//
//    Parameters - none
//
//    Purpose
//    Constructor.  Called during instantiation of MyExchExt object.
//
//
MyExchExt::MyExchExt()
{
  m_cRef = 1;
  m_pExchExtPropertySheets = new MyExchExtPropertySheets(this);
  m_pExchExtMessageEvents = new MyExchExtMessageEvents(this);
  m_pOutlookExtItemEvents = new MyOutlookExtItemEvents(this);
}

///////////////////////////////////////////////////////////////////////////////
//  IExchExt virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtPropertySheets::Release()
//
//    Parameters - none
//
//    Purpose
//    Frees memory when interface is not referenced any more
//
//    Return value
//    reference count of interface
//

STDMETHODIMP_(ULONG) MyExchExt::Release() 
{ 
 ULONG ulCount = --m_cRef;
                         
 if (!ulCount) 
 { 
  delete this;
 }

return ulCount;

}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExt::QueryInterface()
//
//    Parameters
//    riid   -- Interface ID.
//    ppvObj -- address of interface object pointer.
//
//    Purpose
//    Returns a pointer to an interface object that is requested by ID.
//
//    Comments
//    The interfaces are requested everytime a new context is entered.  The 
//    IID_IExchExt* interfaces are ignored if not specified in the Exchange
//    extensions registry.
//
//    If an interface pointer is returned for more than one context, that
//    interface is used by the client for each of those contexts.  Check the
//    current context to verify if it is appropriate to pass back an interface
//    pointer.
//    
//    Return Value - none
//

STDMETHODIMP MyExchExt::QueryInterface(REFIID riid, LPVOID FAR * ppvObj)
{
    HRESULT hResult = S_OK;

    *ppvObj = NULL;

    if (( IID_IUnknown == riid) || ( IID_IExchExt == riid) )
    {
        *ppvObj = (LPUNKNOWN)this;
    }
    else if (IID_IExchExtPropertySheets == riid)
    {
     // if we are in the read or send context, do not return
     // propertysheet interface
        if ( (m_context == EECONTEXT_SENDNOTEMESSAGE)   ||
             (m_context == EECONTEXT_SENDPOSTMESSAGE)   ||
             (m_context == EECONTEXT_SENDRESENDMESSAGE) ||
             (m_context == EECONTEXT_READNOTEMESSAGE)   ||
             (m_context == EECONTEXT_READPOSTMESSAGE)   ||
             (m_context == EECONTEXT_READREPORTMESSAGE) )
                 return E_NOINTERFACE;

     // otherwise return the interface
        *ppvObj = (LPUNKNOWN) m_pExchExtPropertySheets;
    }
    else if (IID_IExchExtMessageEvents == riid)
    {
        *ppvObj = (LPUNKNOWN) m_pExchExtMessageEvents;
	}
	else if (IID_IOutlookExtItemEvents == riid )
	{
		*ppvObj = (LPUNKNOWN) m_pOutlookExtItemEvents;
    }
    else
        hResult = E_NOINTERFACE;

    if (NULL != *ppvObj)
        ((LPUNKNOWN)*ppvObj)->AddRef();

    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExt::Install()
//
//    Parameters
//    peecb     -- pointer to Exchange Extension callback function
//    eecontext -- context code at time of being called.
//
//    Purpose
//    Called once for each new context that is entered.
//
//    Return Value
//    S_OK - the installation succeeded for the context
//    S_FALSE - deny the installation fo the extension for the context
//
STDMETHODIMP MyExchExt::Install(LPEXCHEXTCALLBACK peecb, ULONG eecontext, ULONG ulFlags)
{
    ULONG ulBuildVersion;
    HRESULT hr;

    m_context = eecontext;
       
    // make sure this is the right version 
    peecb->GetVersion(&ulBuildVersion, EECBGV_GETBUILDVERSION);
    if (EECBGV_BUILDVERSION_MAJOR != (ulBuildVersion & 
                                      EECBGV_BUILDVERSION_MAJOR_MASK))
        return S_FALSE;


    switch (eecontext)
    {
     case EECONTEXT_PROPERTYSHEETS:
     case EECONTEXT_SENDNOTEMESSAGE:
     case EECONTEXT_SENDPOSTMESSAGE:
     case EECONTEXT_SENDRESENDMESSAGE:
     case EECONTEXT_READNOTEMESSAGE:
     case EECONTEXT_READPOSTMESSAGE:
     case EECONTEXT_READREPORTMESSAGE:
        hr = S_OK;
        break;

     default:
        hr = S_FALSE;
        break;
    }

	peecb->GetSession ( &lpSession, &lpAdrbook ) ;

    return hr;

}

///////////////////////////////////////////////////////////////////////////////
//  IExchExtPropertySheets virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtPropertySheets::QueryInterface()
//
//    Parameters
//    riid   -- Interface ID.
//    ppvObj -- address of interface object pointer.
//
//    Purpose
//    Return interface object upon request
//    
//    Return Value - none
//
//    Comments
//    Currently the Exchange client does not call QueryInterface from any object
//    except for IExchExt.  This is implemented in case features are added to
//    Exchange to require QueryInterface from any object.  Also, as a "rule of
//    OLE COM" this is the proper implementation of QueryInterface.
//

STDMETHODIMP MyExchExtPropertySheets::QueryInterface(REFIID riid, LPVOID FAR * ppvObj)
{   

    *ppvObj = NULL;
    if (riid == IID_IExchExtPropertySheets)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    if (riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)m_pExchExt;  // return parent interface
        m_pExchExt->AddRef();
        return S_OK;
    }

    return E_NOINTERFACE;

}


///////////////////////////////////////////////////////////////////////////////
//    MyExchExtPropertySheets::GetMaxPageCount()
//
//    Parameters
//    ulFlags -- a bitmask indicating what type of property sheet is being 
//               displayed
//
//    Purpose
//    Returns the number of property pages which are to be added.
//
//    Return Value - maximum number of custom pages for the property sheet
//
//    Exchange calls this to know how many PROPSHEETPAGE buffers it needs
//    to allocate.
//

STDMETHODIMP_ (ULONG) MyExchExtPropertySheets::GetMaxPageCount(ULONG ulFlags)          
{
 ULONG ulNumExtSheets;

    switch (ulFlags)
    {
     // ignore these objects.
     case EEPS_FOLDER:
     case EEPS_STORE:
     case EEPS_MESSAGE:
        ulNumExtSheets = 0;
        break;
         
     case EEPS_TOOLSOPTIONS:
        ulNumExtSheets = 1;   // adding one propery page
        break;

     default:
        ulNumExtSheets = 0;
        break;
    }

    return ulNumExtSheets;  
}


///////////////////////////////////////////////////////////////////////////////
//    MyExchExtPropertySheets::GetPages()
//
//    Parameters
//    peecb   -- pointer to Exchange callback interface
//    ulFlags -- a bitmask indicating what type of property sheet is being 
//               displayed
//    ppsp    -- output parm pointing to pointer to list of property sheets
//    pcpsp   -- output parm pointing to buffer contaiing number of property
//               sheets actually used.
//
//    Purpose
//    Fills the PROPSHEETPAGE members for the custom property page.
//
//    Return Value
//    S_FALSE - do not add a new page
//    S_OK - use the ppsp information for new pages.
//
//    Comments
//    Exchange calls this method to gather information for any custom
//    property pages to be added to the sheet.  Here we are only adding
//    one.  ppsp may be an array of PROPSHEETPAGE structures to allow you
//    to all multiple property pages.
//

STDMETHODIMP MyExchExtPropertySheets::GetPages(LPEXCHEXTCALLBACK peecb,
                      ULONG ulFlags, LPPROPSHEETPAGE ppsp, ULONG FAR * pcpsp)
{
 LPMDB pMDB = NULL;
 LPMESSAGE pItem = NULL;

    *pcpsp = 0;

    // fill out members for the property page
    ppsp[0].dwSize = sizeof (PROPSHEETPAGE);
    ppsp[0].dwFlags = PSP_DEFAULT | PSP_HASHELP;
    ppsp[0].hInstance = ghInstDLL;
    ppsp[0].pszTemplate = MAKEINTRESOURCE(IDD_LIST_MESSAGEHANDLERS);
    ppsp[0].hIcon = NULL;     // not used
    ppsp[0].pszTitle = NULL;  // not used
    ppsp[0].pfnDlgProc = (DLGPROC)ExchangeHookDlgProc;
    ppsp[0].lParam = 0;     
    ppsp[0].pfnCallback = NULL;
    ppsp[0].pcRefParent = NULL; // not used

    *pcpsp = 1;

    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
//    MyExchExtPropertySheets::FreePages()
//
//    Parameters
//    ppsp -- pointer to a pointer to the first of a list of property pages
//    cpsp -- number of custom property pages in the list
//    ulFlags -- type of property page
//    
//    Purpose
//    Free any memory associated to the property sheet.
//
//    Return Value - none
//
//    Comments
//    No parameters are used in this example but the function is used as
//    a signal that the property sheet is going away and so memory may
//    be freed.
//
                         
STDMETHODIMP_ (VOID) MyExchExtPropertySheets::FreePages(LPPROPSHEETPAGE ppsp, ULONG ulFlags, 
                                        ULONG cpsp)          
{
   // not used 
}


///////////////////////////////////////////////////////////////////////////////
//  IExchExtMessageEvents virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::QueryInterface()
//
//    Parameters
//    riid   -- Interface ID.
//    ppvObj -- address of interface object pointer.
//
//    Purpose
//    Return interface object upon request
//    
//    Return Value - none
//
//    Comments
//    Currently the Exchange client does not call QueryInterface from any object
//    except for IExchExt.  This is implemented in case features are added to
//    Exchange to require QueryInterface from any object.  Also, as a "rule of
//    OLE COM" this is the proper implementation of QueryInterface.
//

STDMETHODIMP MyExchExtMessageEvents::QueryInterface(REFIID riid, LPVOID FAR * ppvObj)          
{   

    *ppvObj = NULL;
    if (riid == IID_IExchExtMessageEvents)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    if (riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)m_pExchExt;  // return parent interface
        m_pExchExt->AddRef();
        return S_OK;
    }
    return E_NOINTERFACE;

}


///////////////////////////////////////////////////////////////////////////////
//    RegistryOpen()
//
//    Purpose
//    Opens the registry key for this utility
//
DWORD RegistryOpen( LPSTR szName, OUT HKEY *phkHandle )
{

  DWORD dwDisposition ;

  RegCreateKeyEx (	HKEY_LOCAL_MACHINE,
					szName,
					0,
					"",
        			REG_OPTION_NON_VOLATILE,
					KEY_ALL_ACCESS,
					NULL,
					phkHandle,
					&dwDisposition ) ;
  
  return dwDisposition ;

}

///////////////////////////////////////////////////////////////////////////////
//    RegistryGet()
//
//    Purpose
//    Gets the registry settings
//
void RegistryGet( HKEY hkHandle )
{
	DWORD dwType = REG_DWORD ;
	DWORD dwSize = sizeof(DWORD) ;
    DWORD cbSenderEmailAddressSize = SenderEmailAddressSize ;
	DWORD cbFilePathSize = FilePathSize ;

	RegQueryValueEx ( hkHandle, "ConfirmDialog", 0, &dwType, (PBYTE)&dwConfirmDialog, &dwSize);
	RegQueryValueEx ( hkHandle, "PreventOpen", 0, &dwType, (PBYTE)&dwPreventOpen, &dwSize);
	RegQueryValueEx ( hkHandle, "SenderEmailAddress", 0, &dwType, (LPBYTE)szSenderEmailAddress, &cbSenderEmailAddressSize ) ;
	RegQueryValueEx ( hkHandle, "FilePath", 0, &dwType, (LPBYTE)szFilePath, &cbFilePathSize ) ;
	RegQueryValueEx ( hkHandle, "Debug", 0, &dwType, (PBYTE)&dwDebug, &dwSize ) ;

}

///////////////////////////////////////////////////////////////////////////////
//    RegistrySet()
//
//    Purpose
//    Sets the registry settings
//
void RegistrySet( HKEY hkHandle )
{
	RegSetValueEx ( hkHandle, "ConfirmDialog", 0, REG_DWORD, (PBYTE)&dwConfirmDialog, sizeof(DWORD) );
	RegSetValueEx ( hkHandle, "PreventOpen", 0, REG_DWORD, (PBYTE)&dwPreventOpen, sizeof(DWORD) );
	RegSetValueEx ( hkHandle, "SenderEmailAddress", 0, REG_SZ, (LPBYTE)szSenderEmailAddress, (lstrlen(szSenderEmailAddress) + 1) * sizeof(CHAR)) ;
	RegSetValueEx ( hkHandle, "FilePath", 0, REG_SZ, (LPBYTE)szFilePath, (lstrlen(szFilePath) + 1) * sizeof(CHAR)) ;
	RegSetValueEx ( hkHandle, "Debug", 0, REG_DWORD, (PBYTE)&dwDebug, sizeof(DWORD) );
}

///////////////////////////////////////////////////////////////////////////////
//    RegistryClose()
//
//    Purpose
//    Closes the registry key for this utility
//
void RegistryClose ( HKEY hkHandle )
{
	RegCloseKey ( hkHandle ) ;
}

///////////////////////////////////////////////////////////////////////////////
//    RegistryDelete()
//
//    Purpose
//    Deletes one of the registry subkeys for this utility
//
void RegistryDelete ( HKEY hkHandle, LPCTSTR lpSubKey )
{
	RegDeleteKey ( hkHandle, lpSubKey ) ;
}

///////////////////////////////////////////////////////////////////////////////
//    GetPropBinaryAsString(lpObj,ulPropTag,lpcbProp,lppvProp)
//
//    Parameters
//    lpObj -- pointer to MAPI object
//	  ulPropTag -- property to read
//	  lpcbProp -- size of property read
//    lppvProp -- string to store property in when read
//
//    Return Value
//    HRESULT value:
//       S_OK -- succeeded
//       anything means some internal call failed
//
//    Purpose
//    Reads a binary property of the message
//

HRESULT GetPropBinaryAsString(             
    IN LPMESSAGE pMessage,
    IN ULONG ulPropTag,
    OUT ULONG *lpcbProp,          
    OUT LPVOID *lppvProp)
{
    HRESULT      hr          = NOERROR;
    HRESULT      hrT         = NOERROR;
    ULONG        cValues     = 0;
    LPSPropValue lpPropValue = NULL;
	LPSTR		 lpszPropValue = 0 ;
	ULONG		 cBytes			= 0 ;

	SizedSPropTagArray(1, rgPropTag) =
    {
        1,
        {
            0
        }
    };

    rgPropTag.cValues = 1;
    rgPropTag.aulPropTag[0] = ulPropTag;

    hrT = pMessage->GetProps(
         (LPSPropTagArray)&rgPropTag,
         fMapiUnicode,
         &cValues,
         &lpPropValue);

    if(hrT == MAPI_W_ERRORS_RETURNED)
    {
        if((lpPropValue != NULL) && (lpPropValue->Value.ul == MAPI_E_NOT_FOUND))
        {
            hr = MAPI_E_NOT_FOUND ;
        }
        else
        {
            hr = E_FAIL;
        }
        goto cleanup;
    }

    if(FAILED(hrT))
    {
        lpPropValue = NULL;

        hr = E_FAIL;
        goto cleanup;
    }

    cBytes = (lpPropValue[0].Value.bin.cb + sizeof(TCHAR)) * (2 * sizeof(TCHAR));

    hr = MAPIAllocateBuffer(cBytes, (void **)&lpszPropValue);

    if(FAILED(hr))
    {                                                   
       goto cleanup;                                
    }                                                   

    HexFromBin(lpPropValue[0].Value.bin.lpb, lpPropValue[0].Value.bin.cb, lpszPropValue );

    *lpcbProp = cBytes ;
	*lppvProp = lpszPropValue ;

cleanup:

	//MAPIFREEBUFFER(lpPropValue) ;
	//MAPIFREEBUFFER(lpszPropValue) ;
    return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    GetPropString(lpObj,ulPropTag,lpcbProp,lppvProp)
//
//    Parameters
//    lpObj -- pointer to MAPI object
//	  ulPropTag -- property to read
//	  lpcbProp -- size of property read
//    lppvProp -- string to store property in when read
//
//    Return Value
//    HRESULT value:
//       S_OK -- succeeded
//       anything means some internal call failed
//
//    Purpose
//    Reads a string property of the message
//

HRESULT GetPropString(            
    IN LPMESSAGE pMessage,          
    IN ULONG ulPropTag,           
    OUT ULONG *lpcbProp,          
    OUT LPVOID *lppvProp)         
{
    HRESULT      hr          = NOERROR;
    HRESULT      hrT         = NOERROR;
    ULONG        cValues     = 0;
    LPSPropValue lpPropValue = NULL;
    ULONG        cbProp      = 0;
    SCODE        sc          = 0;

    SizedSPropTagArray(1, rgPropTag) =
    {
        1,
        {
            0
        }
    };

    *lpcbProp = 0;
    *lppvProp = NULL;

    rgPropTag.cValues = 1;
    rgPropTag.aulPropTag[0] = ulPropTag;

    hrT = pMessage->GetProps(
         (LPSPropTagArray)&rgPropTag,
         fMapiUnicode,
         &cValues,
         &lpPropValue);

    if(hrT == MAPI_W_ERRORS_RETURNED)
    {
       if((lpPropValue != NULL) && (lpPropValue->Value.ul == MAPI_E_NOT_FOUND))
        {
            hr = MAPI_E_NOT_FOUND;
        }
        else
        {
           hr = E_FAIL;
        }
        goto cleanup;
    }

    if(FAILED(hrT))
    {
        lpPropValue = NULL;

        hr = E_FAIL;
        goto cleanup;
    }

    if(PROP_TYPE(ulPropTag) == PT_STRING8)
    {
        cbProp = ((lpPropValue->Value.lpszA == NULL) ?
            sizeof(CHAR) :
            cbStrLenA(lpPropValue->Value.lpszA));
    }
    else
    {
        cbProp = ((lpPropValue->Value.lpszW == NULL) ?
            sizeof(WCHAR) :
            cbStrLenW(lpPropValue->Value.lpszW));
    }

    sc = MAPIAllocateBuffer(cbProp, lppvProp);

    if(FAILED(sc))
    {
        hr = E_OUTOFMEMORY;
        goto cleanup;
    }

    if(PROP_TYPE(ulPropTag) == PT_STRING8)
    {
        memcpy(*lppvProp, lpPropValue->Value.lpszA, cbProp);
    }
    else
    {
        memcpy(*lppvProp, lpPropValue->Value.lpszW, cbProp);
    }

    *lpcbProp = cbProp;

cleanup:

   //MAPIFreeBuffer(lpPropValue);
   MAPIFreeBuffer(lppvProp);
   return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    RunApp( )
//
//    Purpose
//    Runs the application
//

void RunApp( LPSTR szCmdArgs )
{
    CHAR		szCmdLine[400] = TEXT ( "" ) ;
	STARTUPINFO	si;
	PROCESS_INFORMATION pi;

	lstrcat(szCmdLine, szFilePath);
	lstrcat(szCmdLine, szSpace ) ;
	lstrcat(szCmdLine, szCmdArgs ) ;

	if ( dwDebug == 1 )
	{
    MessageBox(NULL, 
        (LPSTR)szCmdLine,
         szMessageBoxTitle,
		 MB_ICONEXCLAMATION | MB_OK);
	}

    si.cb = sizeof(STARTUPINFO);
    si.lpReserved = NULL;
    si.lpDesktop = NULL;
    si.lpTitle = NULL;
    si.dwFlags = 0;
    si.cbReserved2 = 0;
    si.lpReserved2 = NULL;

	if ( !CreateProcess (
		NULL,
		(LPTSTR)szCmdLine,
		NULL,
		NULL,
		FALSE,
		CREATE_NEW_CONSOLE | NORMAL_PRIORITY_CLASS ,
		NULL,
		NULL,
		&si,
		&pi))
	{ 
		PVOID lpMsgBuf;
		
		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		    (LPTSTR) &lpMsgBuf,
			0,
			NULL );
		MessageBox( NULL, (LPTSTR)lpMsgBuf, szMessageBoxTitle, MB_OK | MB_ICONINFORMATION );
		
		LocalFree( lpMsgBuf );
 	}

}

///////////////////////////////////////////////////////////////////////////////
//    ProcessMessage(pMessage)
//
//    Parameters
//    pMessage -- pointer to message object
//
//    Purpose
//    Processes the message
//
//    Return Value
//    HRESULT value:
//       S_OK -- succeeded
//       anything means some internal call failed
//

HRESULT ProcessMessage( LPMESSAGE pMessage )
{
  HRESULT	hr = S_OK ;
  LPTSTR  lpszSubject = NULL;
  ULONG   cbSubject = 0;
  ULONG   cbEntryID = 0;
  LPTSTR  lpszEntryID = NULL;
  char		szArgName[40] = TEXT ( "MessageID" ) ;
  char		szArgs[400] = TEXT ( "" ) ;
  
 if ( (pMessage == NULL) )
 {
      hr = MAPI_E_INVALID_PARAMETER;
      goto error_return;
 }

 hr = GetPropBinaryAsString (
	 (LPMESSAGE)pMessage,
	 PR_ENTRYID,
     &cbSubject,
     (LPVOID *)&lpszEntryID);

 if(FAILED(hr))
 {
   goto error_return;
 }

 lstrcat ( szArgs, szQuote );
 lstrcat ( szArgs, szArgName );
 lstrcat ( szArgs, szQuote );
 lstrcat ( szArgs, szSpace );
 lstrcat ( szArgs, szQuote );
 lstrcat ( szArgs, lpszEntryID );
 lstrcat ( szArgs, szQuote );

 RunApp( szArgs ) ;

 error_return:

	return hr;

}

///////////////////////////////////////////////////////////////////////////////
//    CheckMessage(pMessage)
//
//    Parameters
//    lpeecb - Exchange extension callback pointer
//
//    Return Value
//    HRESULT value:
//       S_FALSE -- We don't care about the message, 
//					or we care about it, but still open the message
//       E_ABORT -- We are going to process the message, don't open it
//
//    Purpose
//    Sees if the message is one we want to process
//

HRESULT CheckMessage (LPEXCHEXTCALLBACK lpeecb )
{
  HRESULT	hr;
  HCURSOR	hOldCursor = NULL;
  HWND		hWnd;
  LPMDB		pMDB = NULL;
  LPMESSAGE pMessage ;
  ULONG		cbSenderEmail = 0;

  lpeecb->GetWindow(&hWnd);

  //Get a pointer to the message
  hr = lpeecb->GetObject(&pMDB, (LPMAPIPROP *)&pMessage);
  if (FAILED(hr))
  {
	  ErrorMessageBox(hWnd, hr, szMessageBoxTitle, 
        "An error occured while getting a pointer to the message.");
    goto error_return;
  }
  
  //Get the sender's email address
  hr = GetPropString(
        (LPMESSAGE)pMessage,
        PR_SENDER_EMAIL_ADDRESS,
        &cbSenderEmail,
        (LPVOID *)&lpszSenderEmail);

  if(FAILED(hr))
  {
    goto error_return ;
  }
  
  //See if it's for one of the handlers
  hr = RegistryFindHandler () ;

  //Some error
  if(FAILED(hr))
  {
    goto error_return ;
  }

  //Not a message we need to handle
  if(hr == S_FALSE)
  {
    goto error_return ;
  }

  //See if the user wanted a dialog box
  if ( dwConfirmDialog == 1 )
  {
  int nRet = MessageBox ( hWnd, 
	"Do you want to process this document?",
	szMessageBoxTitle, MB_YESNO);
	if (nRet == IDNO)
	{
	  hr = S_FALSE ;
	  goto error_return ;
	}
  }

  //Process the message
  hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
  hr = ProcessMessage(pMessage);
  if (FAILED(hr))
  {
  ErrorMessageBox(hWnd, hr, szMessageBoxTitle, 
    "An error occured while processing the message.");
  hr = S_FALSE ;
  goto error_return;
  }
  
  //See if we are still supposed to open the message
  //This tells Exchange/Outlook to allow the message to open
  hr = S_FALSE ;
  if ( dwPreventOpen == 1 )
  {
	//this will tell Exchange/Outlook to not display the read note UI at all.
	hr = E_ABORT;
  }
  return hr ;

  error_return:

    UlRelease(pMDB);

    UlRelease(pMessage);

    MAPIFreeBuffer(lpszSenderEmail);

    if(hOldCursor)
	{
      SetCursor(hOldCursor);
	}

    return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnRead()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    To extend or inhibit Exchange when displaying the send or read note form.
//    
//    Return Value
//    S_OK signals Exchange to not continue calling extensions
//    S_FALSE signals Exchange to continue calling extensions
//    Other MAPI Code errors will abort the send or read note form.
//
//

STDMETHODIMP MyExchExtMessageEvents::OnRead(LPEXCHEXTCALLBACK lpeecb)
{
  HRESULT hr;

  hr = S_FALSE; 

  return hr;
  //HRESULT hr;
  //
  //hr = CheckMessage(lpeecb,
  //	  dwProcessOnRead,
  //	  dwDialogOnRead );
  //
  //return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnReadComplete()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    To do processing after message has been read.
//    
//    Return Value
//    S_OK signals Exchange to not continue calling extensions
//    S_FALSE signals Exchange to continue calling extensions
//    Some MAPI Code error indicates a problem and will not display the send
//    or read note form.
//
//    Comments.
//    If an error code, such as MAPI_E_CALL_FAILED, is returned, Exchange will
//    call OnReadComplete again with the ulFlags parameter set to 
//    EEME_COMPLETE_FAILED.  Returning the error code again will cause Exchange
//    to not display the UI.
//

STDMETHODIMP MyExchExtMessageEvents::OnReadComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{
  HRESULT hr;

  hr = S_FALSE; 

  return hr;
  //HRESULT hr;
  //
  //hr = CheckMessage(lpeecb,
  //  dwProcessOnReadComplete,
  //	  dwDialogOnReadComplete );
  //
  //return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnWrite()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    This method is called when a message is about to be written.  The message
//    only has default properties at this point.  It does not contain 
//    properties which the user has added by way of recipients, subject,
//    message text, or attachments.
//    This method is called when the user Sends or Saves a message
//    
//    Return Value
//    S_OK signals Exchange to not continue calling extensions
//    S_FALSE signals Exchange to continue calling extensions
//
//

STDMETHODIMP MyExchExtMessageEvents::OnWrite(LPEXCHEXTCALLBACK lpeecb)
{
  HRESULT hr;

  hr = S_FALSE; 

  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnWriteComplete()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    This method is called when the data (recipients, attachments, body, 
//    subject, etc.) has been written to the message.
//    
//    Return Value
//    S_OK signals Exchange to not continue calling extensions
//    S_FALSE signals Exchange to continue calling extensions
//
//

STDMETHODIMP MyExchExtMessageEvents::OnWriteComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{

	// Not used
	
  HRESULT hr;
  
  hr = S_FALSE ;

  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnCheckNames()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called when user selects the Check Names button and just before message
//    is submitted to MAPI.
//    
//    Return Value
//    S_OK signals Exchange to not continue calling extensions
//    S_FALSE signals Exchange to continue calling extensions
//

STDMETHODIMP MyExchExtMessageEvents::OnCheckNames(LPEXCHEXTCALLBACK lpeecb)
{
  // Not used

	HRESULT		hr ;

	hr = S_FALSE ;

	return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnCheckNamesComplete()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called after exchange has completed resolving names in the message
//    recipients table.
//    
//    Return Value
//    S_OK signals Exchange to not continue calling extensions
//    S_FALSE signals Exchange to continue calling extensions
//

STDMETHODIMP MyExchExtMessageEvents::OnCheckNamesComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{
  // Not used

	HRESULT		hr ;

	hr = S_FALSE ;

	return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnSubmit()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called before message data has been written and is
//    is submitted to MAPI.
//    
//    Return Value
//    S_OK signals Exchange to not continue calling extensions
//    S_FALSE signals Exchange to continue calling extensions
//
//

STDMETHODIMP MyExchExtMessageEvents::OnSubmit(LPEXCHEXTCALLBACK lpeecb)
{
  HRESULT hr;

  hr = S_FALSE;   

  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    MyExchExtMessageEvents::OnSubmitComplete()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called after message has been submitted to MAPI.
//    
//    Return Value - none
//
//

STDMETHODIMP_ (VOID) MyExchExtMessageEvents::OnSubmitComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{
	// not used
}

///////////////////////////////////////////////////////////////////////////////
//  Helper Functions
//

///////////////////////////////////////////////////////////////////////////////
//    ErrorMessageBox()
//
//    Parameters
//    hWnd       -- parent window
//    hr         -- HRESULT value
//    szFunction -- function name in which the error occurred
//    szMessage  -- error message
//
//    Purpose
//    Displays an error message using MessageBox
//
//    Return Value - none
//
//    Comments
//    Pass 0 for hr to not display an error number.  Pass NULL for
//    szFunction to not display a function name.  Use these options
//    to display error messages for public consumption.  Use hr and
//    function name for internal errors and for debugging.  szMessage
//    is mandatory.
//
void ErrorMessageBox(HWND hWnd, HRESULT hr, LPSTR szFunction, LPSTR szMessage)
{
 static char szError[256];

 if (szMessage == NULL)
 {
    MessageBox(hWnd, 
         "An unknown error occured.",
         szMessageBoxTitle,
		 MB_ICONEXCLAMATION | MB_OK);
    return;
 }

 if ((hr == 0) && (szFunction == NULL))
 {
    MessageBox(hWnd,
		szMessage,
		szMessageBoxTitle,
		MB_ICONEXCLAMATION | MB_OK);
    return;
 }

 if (szFunction != NULL) 
 {
  wsprintf(szError, "Error %08X in %s\n%s", hr, szFunction, szMessage);
  MessageBox(hWnd, szError, szMessageBoxTitle, MB_ICONEXCLAMATION | MB_OK);
 }

}

///////////////////////////////////////////////////////////////////////////////
//    RegistryEnumKeys()
//
//    Purpose
//    Gets the registry keys
//
void RegistryEnumKeys( HWND HDlg, HKEY hkHandle )
{
	char		szName[MAX_PATH];
	char		szClass[MAX_PATH];
	DWORD		cbClassSize ;
	DWORD		cbSubKeys;
	DWORD		cbMaxSubKey;
	DWORD		cbMaxClass;
	DWORD		cbValues;
	DWORD		cbMaxValue;
	DWORD		cbMaxValueData ;
	DWORD		cbSecurityDescriptor ;
	DWORD		i ;
	DWORD		retCode ;
	FILETIME	ftLastWriteTime ;

	//Find out how many subkeys there are
    RegQueryInfoKey ( hkHandle,
		szClass,
		&cbClassSize,
		NULL,
		&cbSubKeys,
		&cbMaxSubKey,
		&cbMaxClass,
		&cbValues,
		&cbMaxValue,
		&cbMaxValueData,
		&cbSecurityDescriptor,
		&ftLastWriteTime ) ;

	for ( i=0, retCode = ERROR_SUCCESS; retCode == ERROR_SUCCESS; i++ )
	{
		retCode = RegEnumKey ( hkHandle, i, szName, MAX_PATH ) ;
		if ( retCode == (DWORD)ERROR_SUCCESS)
		{
			SendMessage ( GetDlgItem ( HDlg, IDC_LIST1 ), LB_ADDSTRING, 0, (LONG) szName ) ;
		}
	}
	//Select the first item by default
	SendMessage ( GetDlgItem ( HDlg, IDC_LIST1 ), LB_SETCURSEL, 0, 0 ) ;
}

///////////////////////////////////////////////////////////////////////////////
//    RegistryList()
//
//    Purpose
//    Gets the defined handlers
//
void RegistryList( HWND HDlg )
{
	HKEY    hkHandle;
	DWORD   dwDisposition;

	dwDisposition = RegistryOpen ( szRootKey, &hkHandle ) ;

	if ( dwDisposition != REG_CREATED_NEW_KEY )
	{
		RegistryEnumKeys ( HDlg, hkHandle ) ;
	}
	RegistryClose ( hkHandle ) ;
}

///////////////////////////////////////////////////////////////////////////////
//    RegistryFindHandler()
//
//    Purpose
//    Finds the Message Handler for a message (if any)
//
HRESULT RegistryFindHandler()
{
	char		szName[MAX_PATH];
	char		szClass[MAX_PATH];
	DWORD		dwType = REG_DWORD ;
	DWORD		cbSenderEmailAddressSize = SenderEmailAddressSize ;
	DWORD		cbClassSize ;
	DWORD		cbSubKeys;
	DWORD		cbMaxSubKey;
	DWORD		cbMaxClass;
	DWORD		cbValues;
	DWORD		cbMaxValue;
	DWORD		cbMaxValueData ;
	DWORD		cbSecurityDescriptor ;
	DWORD		i ;
	DWORD		retCode ;
	FILETIME	ftLastWriteTime ;
	HKEY		hkHandle;
	HKEY		hkHandlerKey;
	char		szHandlerKey[120] = TEXT ( "" ) ;
	char		szHandlerEmailAddress[SenderEmailAddressSize] = TEXT ( "" ) ;

	//Open the registry item for the handler
	RegistryOpen ( szRootKey,
		&hkHandle ) ;

	//Find out how many subkeys there are
    RegQueryInfoKey ( hkHandle,
		szClass,
		&cbClassSize,
		NULL,
		&cbSubKeys,
		&cbMaxSubKey,
		&cbMaxClass,
		&cbValues,
		&cbMaxValue,
		&cbMaxValueData,
		&cbSecurityDescriptor,
		&ftLastWriteTime ) ;

	//Loop through the subkeys
	for ( i = cbSubKeys - 1, retCode = ERROR_SUCCESS; retCode == ERROR_SUCCESS; i-- )
	{
		retCode = RegEnumKey ( hkHandle, i, szName, MAX_PATH ) ;
		if ( retCode == (DWORD)ERROR_SUCCESS)
		{

			//Open the subkey
			lstrcpy(szHandlerKey, szRootKey) ;
			lstrcat(szHandlerKey, "\\") ;
			lstrcat(szHandlerKey, szName) ;
			lstrcat(szHandlerKey, "\0") ;

			RegistryOpen ( szHandlerKey,
				&hkHandlerKey ) ;

			//Get the email address for the subkey
			RegQueryValueEx ( hkHandlerKey,
				"SenderEmailAddress",
				0, &dwType,
				(LPBYTE)szHandlerEmailAddress,
				&cbSenderEmailAddressSize ) ;

			//See if it is the same as the email address
			//for the message
			if (lstrcmpi(lpszSenderEmail,szHandlerEmailAddress) == 0)
			{
				//if so, read the other data for the handler and exit
				RegistryGet ( hkHandlerKey ) ;
				RegistryClose ( hkHandlerKey ) ;	
				RegistryClose ( hkHandle ) ;
				return S_OK ;
			}
			//Close the registry for this subkey
			RegistryClose ( hkHandlerKey ) ;
		}
	}
	RegistryClose ( hkHandle ) ;
	return S_FALSE ;
}

STDMETHODIMP MyOutlookExtItemEvents::QueryInterface(REFIID riid, LPVOID FAR * ppvObj)          
{   

    *ppvObj = NULL;
    if (riid == IID_IOutlookExtItemEvents)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    if (riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)m_pExchExt;  // return parent interface
        m_pExchExt->AddRef();
        return S_OK;
    }
    return E_NOINTERFACE;

}

///////////////////////////////////////////////////////////////////////////////
//    IOutlookExtItemEvents::OnOpen()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called before Outlook actually opens an item for viewing
//    Unlike OnRead, this event is not fired on a preview
//    
//    Return Value
//    E_ABORT signals Outlook/Exchange to not continue calling extensions
//    S_FALSE signals Outlook/Exchange to continue calling extensions
//

STDMETHODIMP MyOutlookExtItemEvents::OnOpen(LPEXCHEXTCALLBACK lpeecb)
{
  HRESULT hr;
  hr = CheckMessage(lpeecb);
  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    IOutlookExtItemEvents::OnOpenComplete()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called after Outlook actually opens an item for viewing
//    Unlike OnReadComplete, this event is not fired on a preview
//    
//    Return Value
//    E_ABORT signals Outlook/Exchange to not continue calling extensions
//    S_FALSE signals Outlook/Exchange to continue calling extensions
//

STDMETHODIMP MyOutlookExtItemEvents::OnOpenComplete(LPEXCHEXTCALLBACK lpeecb,ULONG ulFlags)
{
	//Not used
	HRESULT		hr ;
	hr = S_FALSE ;
	return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    IOutlookExtItemEvents::OnClose()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called before Outlook closes an open item
//    
//    Return Value
//    E_ABORT signals Outlook/Exchange to not continue calling extensions
//    S_FALSE signals Outlook/Exchange to continue calling extensions
//
           
STDMETHODIMP MyOutlookExtItemEvents::OnClose(LPEXCHEXTCALLBACK lpeecb,ULONG ulSaveOptions)
{
	//Not used
	HRESULT		hr ;
	hr = S_FALSE ;
	return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    IOutlookExtItemEvents::OnCloseComplete()
//
//    Parameters
//    lpeecb -- pointer to IExchExtCallback interface
//
//    Purpose
//    Called after Outlook closes an open item
//    
//    Return Value
//    E_ABORT signals Outlook/Exchange to not continue calling extensions
//    S_FALSE signals Outlook/Exchange to continue calling extensions
//
          
STDMETHODIMP MyOutlookExtItemEvents::OnCloseComplete(LPEXCHEXTCALLBACK lpeecb,ULONG ulFlags)
{
	//Not used
	HRESULT		hr ;
	hr = S_FALSE ;
	return hr;
}



