///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      EXCHHOOK.H
//
//  Description
//      Interface Methods are declared in this file for exchhook.cpp
//
//  Author
//      Bruce Armstrong
//
//  Revision: 1.00
//
// Copyright (c) 1998 All rights reserved.
//
#ifndef __EXCHHOOK_H__
#define __EXCHHOOK_H__

#include <WINDOWS.H> 
#include <COMMCTRL.H>
#include <MAPIX.H>
#include <MAPIUTIL.H>
#include <MAPIFORM.H>
#include <EXCHEXT.H>
#include "RESOURCE.H"
#include <EDKMDB.H>
#include "OUTLEXT.H"

///////////////////////////////////////////////////////////////////////////////
//    Prototypes
//

extern "C"
{
 LPEXCHEXT CALLBACK ExchEntryPoint(void);
}

void ErrorMessageBox(HWND hWnd, HRESULT hr, LPSTR szFunction, LPSTR szMessage);
DWORD RegistryOpen( LPSTR szName, OUT HKEY *phkHandle ) ;
void RegistryGet ( HKEY hkHandle );
void RegistrySet ( HKEY hkHandle );
void RegistryClose ( HKEY hkHandle ) ;
void RegistryDelete ( HKEY hkHandle, LPCTSTR lpSubKey ) ;
void RegistryList ( HWND HDlg ) ;
HRESULT RegistryFindHandler () ;
BOOL CALLBACK ExchangeHookDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

///////////////////////////////////////////////////////////////////////////////
//    Global Data Declarations
//
#define SenderEmailAddressSize 80
#define FilePathSize 128
#define HandlerNameSize 40

extern DWORD dwPreventOpen;
extern DWORD dwConfirmDialog;
extern DWORD dwDebug;

extern char szSenderEmailAddress[];
extern char szFilePath[];

extern LPADRBOOK lpAdrbook ;
extern HINSTANCE ghInstDLL ;

extern char szRootKey[];
extern char  szHandlerName[] ;
extern char  szMessageBoxTitle[] ;

///////////////////////////////////////////////////////////////////////////////
//    forward declarations
//
class MyExchExt;
class MyExchExtPropertySheets;
class MyExchExtMessageEvents;
class MyOutlookExtItemEvents;

class MyExchExt : public IExchExt
{
 
 public:
    MyExchExt(); 
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    STDMETHODIMP_(ULONG) Release(); 
    STDMETHODIMP Install (LPEXCHEXTCALLBACK pmecb, 
                        ULONG mecontext, ULONG ulFlags);
    
 private:
    ULONG m_cRef;
    UINT  m_context;
    MyExchExtPropertySheets * m_pExchExtPropertySheets;
    MyExchExtMessageEvents * m_pExchExtMessageEvents;
    MyOutlookExtItemEvents * m_pOutlookExtItemEvents;
};

class MyExchExtPropertySheets : public IExchExtPropertySheets
{
 public:
    MyExchExtPropertySheets (LPUNKNOWN pParentInterface) { 
    m_pExchExt = pParentInterface;
    m_cRef = 0; 
    };
    
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef;
                         if (!ulCount) { delete this; }
                         return ulCount;};

    STDMETHODIMP_ (ULONG) GetMaxPageCount(ULONG ulFlags);          
    STDMETHODIMP  GetPages(LPEXCHEXTCALLBACK peecb,
                          ULONG ulFlags, LPPROPSHEETPAGE ppsp, ULONG FAR * pcpsp);          
    STDMETHODIMP_ (VOID) FreePages(LPPROPSHEETPAGE ppsp, ULONG ulFlags, 
                                         ULONG cpsp);          

 private:
    ULONG m_cRef;
    LPUNKNOWN m_pExchExt;
};

class MyExchExtMessageEvents : public IExchExtMessageEvents
{
 public:
    MyExchExtMessageEvents (LPUNKNOWN pParentInterface) { 
    m_pExchExt = pParentInterface;
    m_cRef = 0; 
    };

    STDMETHODIMP QueryInterface
                    (REFIID riid,
                     LPVOID * ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef;
                         if (!ulCount) { delete this; }
                         return ulCount;};

    STDMETHODIMP OnRead(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP OnReadComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
    STDMETHODIMP OnWrite(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP OnWriteComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
    STDMETHODIMP OnCheckNames(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP OnCheckNamesComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
    STDMETHODIMP OnSubmit(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP_ (VOID) OnSubmitComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
 private:
    ULONG		m_cRef;
	LPUNKNOWN	m_pExchExt;

};

class MyOutlookExtItemEvents : public IOutlookExtItemEvents
{
 public:
    MyOutlookExtItemEvents (LPUNKNOWN pParentInterface) { 
    m_pExchExt = pParentInterface;
    m_cRef = 0; 
    };

    STDMETHODIMP QueryInterface
                    (REFIID riid,
                     LPVOID * ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef;
                         if (!ulCount) { delete this; }
                         return ulCount;};

    STDMETHODIMP OnOpen(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP OnOpenComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
    STDMETHODIMP OnClose(LPEXCHEXTCALLBACK lpeecb, ULONG ulSaveOptions);
    STDMETHODIMP OnCloseComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
   
 private:
    ULONG		m_cRef;
	LPUNKNOWN	m_pExchExt;

};


#endif  // __EXCHHOOK_H__
